@@ -19,27 +19,39 @@ import itertools
 import logging
 
 
-__version__ = '0.9.3'
+__version__ = '0.11.0'
 
 
 try:
     import matplotlib.pyplot as plt
     import matplotlib.patches as pltp
 
-    def bubbles(circles, labels, lim=None):
+    def get_default_label(count, circle):
+        """Generates a default label."""
+        if circle.ex and 'id' in circle.ex:
+            label = str(circle.ex['id'])
+        elif circle.ex and 'datum' in circle.ex:
+            label = circle.ex['datum']
+        elif circle.ex:
+            label = str(circle.ex)
+        else:
+            label = '#' + str(count)
+        return label
+
+    def bubbles(circles, labels=None, lim=None):
         """Debugging function displays circles with matplotlib."""
         fig, ax = plt.subplots(figsize=(8.0, 8.0))
-        n_missing_labels = len(circles) - len(labels)
-        if n_missing_labels > 0:
-            labels += [''] * n_missing_labels
+        if not labels:
+            labels = [get_default_label(i, c) for i, c in enumerate(circles)]
         for circle, label in zip(circles, labels):
-            x, y, r = circle
+            x, y, r = circle.circle
             ax.add_patch(pltp.Circle((x, y), r, alpha=0.2,
                                      linewidth=2, fill=False))
             ax.text(x, y, label)
         if lim is None:
-            lim = max([max(abs(c.x) + c.r, abs(c.y) + c.r)
-                       for c in circles])
+            lim = max([max(abs(circle.circle.x) + circle.circle.r,
+                           abs(circle.circle.y) + circle.circle.r)
+                       for circle in circles])
         plt.xlim(-lim, lim)
         plt.ylim(-lim, lim)
         plt.show()
@@ -47,13 +59,68 @@ except ImportError:
     pass
 
 
+_Circle = collections.namedtuple('_Circle', ['x', 'y', 'r'])
+FieldNames = collections.namedtuple('Field', ['id', 'datum', 'children'])
+
+class Circle:
+    """Hierarchy element.
+
+    Used as an intermediate and output data structure.
+
+    """
+
+    __slots__ = ['circle', 'level', 'ex']
+
+    def __init__(self, x=0.0, y=0.0, r=1.0, level=1, ex=None):
+        """Initialize Output data structure.
+
+        Args:
+            x (float): x coordinate of the circle center.
+            y (float): y coordinate of the circle center.
+            r (float): radius of the circle.
+            level (int): depth level of the data for hierarchy representation
+                where 0 is the root of the hierarchy.
+            ex (dict): additional data to be carried by the structure (e.g.
+                label, name, parent_id, ...)
+
+        """
+        self.circle = _Circle(x, y, r)
+        self.level = level
+        self.ex = ex
+
+    def __lt__(self, other):
+        """Reversed level order, then normal ordering on datum."""
+        return (self.level, self.r) < (other.level, other.r)
+
+    def __eq__(self, other):
+        """Compare level and datum. No order on id, children and circle."""
+        return (self.level, self.circle, self.ex) == \
+               (other.level, other.circle, other.ex)
+
+    def __repr__(self):
+        """Representation of Output"""
+        return "{}(x={}, y={}, r={}, level={}, ex={!r})".\
+            format(self.__class__.__name__, self.x, self.y, self.r,
+                   self.level, self.ex)
+
+    @property
+    def x(self):
+        return self.circle.x
+
+    @property
+    def y(self):
+        return self.circle.y
+
+    @property
+    def r(self):
+        return self.circle.r
+
+
 log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
 
 _eps = sys.float_info.epsilon
 
-Circle = collections.namedtuple('Circle', ['x', 'y', 'r'])
-
 def distance(circle1, circle2):
     """Compute distance between 2 cirlces."""
     x1, y1, r1 = circle1
@@ -116,17 +183,18 @@ def get_placement_candidates(radius, c1, c2, margin):
         A pair of candidate cirlces where one or both value can be None.
 
     """
-    ic1 = Circle(c1.x, c1.y, c1.r + (radius + margin))
-    ic2 = Circle(c2.x, c2.y, c2.r + (radius + margin))
+    margin = 10.0 * sys.float_info.epsilon
+    ic1 = _Circle(c1.x, c1.y, c1.r + (radius + margin))
+    ic2 = _Circle(c2.x, c2.y, c2.r + (radius + margin))
     i1, i2 = get_intersection(ic1, ic2)
     if i1 is None:
         return None, None
     i1_x, i1_y = i1
-    candidate1 = Circle(i1_x, i1_y, radius)
+    candidate1 = _Circle(i1_x, i1_y, radius)
     if i2 is None:
         return candidate1, None
     i2_x, i2_y = i2
-    candidate2 = Circle(i2_x, i2_y, radius)
+    candidate2 = _Circle(i2_x, i2_y, radius)
     return candidate1, candidate2
 
 
@@ -168,7 +236,7 @@ def pack_A1_0(data):
         data: sorted (descending) list of value to circlify.
 
     Returns:
-        list of circlify.Circle.
+        list of circlify.Output.
 
     """
     assert data == sorted(data, reverse=True), 'data must be sorted (desc)'
@@ -179,7 +247,7 @@ def pack_A1_0(data):
         # Place first 2 circles on each side of (0, 0)
         if n_circles <= 1:
             x = radius if n_circles == 0 else -radius
-            circle = Circle(x, float(0.0), radius)
+            circle = _Circle(x, float(0.0), radius)
             placed_circles.append(circle)
             continue
         mhd = None
@@ -191,14 +259,11 @@ def pack_A1_0(data):
                     # Ignore candidates that overlap with any placed circle.
                     other_placed_circles = [circ for circ in placed_circles
                                             if circ not in [c1, c2]]
-                    overlap = False
-                    for pc in other_placed_circles:
-                        dist =  distance(pc, cand)
-                        if dist < -_eps:
-                            overlap = True
-                            break
-                    if overlap:
-                        continue
+                    if other_placed_circles:
+                        min_dist = min([distance(pc, cand)
+                                        for pc in other_placed_circles])
+                        if min_dist < -_eps:
+                            continue
                     hd = get_hole_degree(cand, placed_circles, c1, c2)
                     if mhd is None or hd > mhd:
                         mhd = hd
@@ -212,12 +277,12 @@ def pack_A1_0(data):
 def extendBasis(B, p):
     """Extend basis to ...  """
     if enclosesWeakAll(p, B):
-        return [p];
+        return [p]
 
     # If we get here then B must have at least one element.
     for i in range(len(B)):
         if enclosesNot(p, B[i]) and enclosesWeakAll(encloseBasis2(B[i], p), B):
-            return [B[i], p];
+            return [B[i], p]
 
     # If we get here then B must have at least two elements.
     for i in range(len(B) - 1):
@@ -226,7 +291,7 @@ def extendBasis(B, p):
                     enclosesNot(encloseBasis2(B[i], p), B[j]) and \
                     enclosesNot(encloseBasis2(B[j], p), B[i]) and \
                     enclosesWeakAll(encloseBasis3(B[i], B[j], p), B):
-                return [B[i], B[j], p];
+                return [B[i], B[j], p]
     raise RuntimeError('If we get here then something is very wrong')
 
 
@@ -267,7 +332,7 @@ def encloseBasis2(a, b):
     y21 = y2 - y1
     r21 = r2 - r1
     l = math.sqrt(x21 * x21 + y21 * y21);
-    return Circle((x1 + x2 + x21 / l * r21) / 2,
+    return _Circle((x1 + x2 + x21 / l * r21) / 2,
                   (y1 + y2 + y21 / l * r21) / 2,
                   (l + r1 + r2) / 2)
 
@@ -297,36 +362,26 @@ def encloseBasis3(a, b, c):
         r = -(B + math.sqrt(B * B - 4 * A * C)) / (2 * A)
     else:
         r = -C / B
-    return Circle(x1 + xa + xb * r, y1 + ya + yb * r, r)
+    return _Circle(x1 + xa + xb * r, y1 + ya + yb * r, r)
 
 
-def scale(circles, target, enclosure=None):
-    """Scale circles in enclosure to fit in the target circle.
+def scale(circle, target, enclosure):
+    """Scale circle in enclosure to fit in the target circle.
 
     Args:
-        circles: Circle objects to scale.
+        circle: Circle to scale.
         target: target Circle to scale to.
         enclosure: allows one to specify the enclosure.
 
     Returns:
-        scaled circles
+        scaled circle
 
     """
-    if not circles:
-        return circles
-    if not enclosure:
-        enclosure = enclose(circles)
-    if enclosure is None:
-        raise ValueError('cannot enclose circles')
     r = target.r / enclosure.r
     t_x, t_y = target.x, target.y
     e_x, e_y = enclosure.x, enclosure.y
-    scaled = []
-    for circle in circles:
-        c_x, c_y, c_r = circle
-        scaled.append(Circle((c_x - e_x) * r + t_x,
-                             (c_y - e_y) * r + t_y, c_r * r))
-    return scaled
+    c_x, c_y, c_r = circle
+    return _Circle((c_x - e_x) * r + t_x, (c_y - e_y) * r + t_y, c_r * r)
 
 
 def enclose(circles):
@@ -342,7 +397,7 @@ def enclose(circles):
     n = len(circles)
     i = 0
     while i < n:
-        p = circles[i];
+        p = circles[i]
         if e is not None and enclosesWeak(e, p):
             i = i + 1
         else:
@@ -352,24 +407,100 @@ def enclose(circles):
     return e
 
 
-# FIXME: remove with_enclosure because it complicates the interface.
-def circlify(data, target_enclosure=None, with_enclosure=False):
+def _handle(data, level, fields=None):
+    """Converts possibly heterogeneous list of float or dict in list of Output.
+
+    Return:
+        list of list of Output. There is one list per level and the (level
+        specific) sub-list sorts data by descending order.
+
+    """
+    if fields is None:
+        fields = FieldNames(None, None, None)
+    datum_field = fields.datum if fields.datum else 'datum'
+    id_field = fields.id if fields.id else 'id'
+    child_field = fields.children if fields.children else 'children'
+    elements = []
+    for datum in data:
+        if isinstance(datum, dict):
+            value = datum[datum_field]
+            elements.append(Circle(r=value + 0, level=level, ex=datum))
+        else:
+            try:
+                elements.append(Circle(r=datum + 0, level=level, ex={"datum": datum}))
+            except TypeError:  # if it fails, assume dict.
+                raise TypeError('dict or numeric value expected')
+    return sorted(elements, reverse=True)
+
+
+def _circlify_level(data, target_enclosure, fields, level=1):
+    """Pack and enclose circles whose radius is linked to the input data.
+
+    All the elements of data are expected to be for the same parent circle
+    called enclosure.
+
+    Args:
+        elements (list of Circle): structured data to be process.
+        target_enclosure (Circle): target enclosure to fit the cirlces into.
+        fields (FieldNames): field names.
+        level (int): level of depth in the hierarchy.
+
+    Returns:
+        list of circlify.Output as value for element of data.
+
+    """
+    all_circles = []
+    if not data:
+        return all_circles
+    circles = _handle(data, 1, fields)
+    packed = pack_A1_0([circle.r for circle in circles])
+    enclosure = enclose(packed)
+    assert enclosure is not None
+    for circle, inner_circle in zip(circles, packed):
+        circle.level = level
+        circle.circle = scale(inner_circle, target_enclosure, enclosure)
+        if circle.ex and fields.children in circle.ex:
+            all_circles += _circlify_level(circle.ex[fields.children],
+                                           circle.circle, fields, level + 1)
+        all_circles.append(circle)
+    return all_circles
+
+
+def _flatten(elements, flattened):
+    """Flattens the elements hierarchy."""
+    if elements is None:
+        return
+    for elem in elements:
+        _flatten(elem.children, flattened)
+        elem.children = None
+        flattened.append(elem)
+    return flattened
+
+
+def circlify(data, target_enclosure=None, show_enclosure=False,
+             datum_field='datum', id_field='id', children_field='children'):
     """Pack and enclose circles whose radius is linked to the input data.
 
     Args:
         data: sorted (descending) array of values.
-        target_enclosure: target ciriclify.Circle where circles should fit in.
-        with_enclosure: appends the target circle to the output if True.
+        target_enclosure: target circlify.Circle where circles should fit in.
+            Defaults to unit circle centered on (0, 0).
+        show_enclosure: insert the target enclosure to the output if True.
+        datum_field: field name that contains the float value when the element is
+            a dict.
+        id_field: field name that contains the id when the element is a dict.
+        children_field: field name that contains the children list when the
+            element is a dict.
 
     Returns:
-        list of circligy.Circle as value for element of data.
+        list of circlify.Circle sorted by ascending level (root to leaf) and
+        descending value (biggest circles first).
 
     """
-    packed = pack_A1_0(data)
+    fields = FieldNames(id=id_field, datum=datum_field, children=children_field)
     if target_enclosure is None:
-        target_enclosure = Circle(float(0.0), float(0.0), float(1.0))
-    packed_and_scaled = scale(packed, target_enclosure)
-    if with_enclosure:
-        packed_and_scaled.append(target_enclosure)
-    return packed_and_scaled
-
+        target_enclosure = Circle(level=0, x=0.0, y=0.0, r=1.0)
+    all_circles = _circlify_level(data, target_enclosure, fields)
+    if show_enclosure:
+        all_circles.append(target_enclosure)
+    return sorted(all_circles)
